from collections import defaultdict
import random

def first(collection, predicate=lambda i: i, default=None):
    return next((x for x in collection if predicate(x)), default)

def groupby(collection, key):
    result = defaultdict(lambda: ())
    for item in collection:
        result[key(item)] += (item,)
    return result

def indexby(collection, key):
    return dict((key(item), item) for item in collection)

def count(collection):
    result = 0
    for item in collection:
        result += 1
    return result

def count_distinct(collection, predicate):
    return len(set([predicate(item) for item in collection]))

def countby(collection, predicate):
    result = defaultdict(int)
    for item in collection:
        result[predicate(item)] += 1
    return result

def partition(list, parts):
    pos = 0
    size = len(list) // parts
    remainder = len(list) % parts
    for i in range(0, parts):
        if i < remainder:
            yield list[pos:pos+size + 1]
            pos += size + 1
        else:
            yield list[pos:pos+size]
            pos += size

def cv_folds(data, n):
    data = list(data)
    random.shuffle(data)
    folds = list(partition(data, n))
    for test_data in folds:
        training_data = []
        for candidate_training_partition in folds:
            if candidate_training_partition != test_data:
                training_data.extend(candidate_training_partition)
        yield (training_data, test_data)

def stratefied_cv_folds(class_lists, n):
    class_list_folds = [cv_folds(class_list, n) for class_list in class_lists]
    for _ in range(0, n):
        training_data = []
        test_data = []
        for class_list_fold in class_list_folds:
            tr, tt = next(class_list_fold)
            training_data.extend(tr)
            test_data.extend(tt)
        yield training_data, test_data

def stratefied_split(class_lists, percent):
    first = []
    second = []
    for l in class_lists:
        l = list(l)
        random.shuffle(l)
        first.extend(l[0:int(len(l) * percent)])
        second.extend(l[int(len(l) * percent):])
    return first, second

def mask(list, mask):
    return tuple(value for value, mask_bit in zip(list, mask) if mask_bit)

def subsequence(list, indexes):
    return [list[index] for index in indexes]

def tupleize(value):
    if isinstance(value, tuple):
        return value
    else:
        return (value,)

def percent_true(items):
    return count(filter(lambda i: i, items)) / len(items)

class IndexMap:
    def __init__(self, list, predicate):
        self.indexes = [index for index, item in enumerate(list) if predicate(item)]

    def __getitem__(self, index):
        return self.indexes[index]