import pdfminer.settings
pdfminer.settings.STRICT = False
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfparser import PDFParser
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams, LTTextBox, LTTextLine, LTFigure, LTChar, LTAnno, LTTextLineHorizontal

class TextExtractor:
    def __init__(self, row_resolution = 5):
        self.row_resolution = row_resolution

    def extract_from_file(self, filename):
        def get_char_objects(parent):
            last_child = None
            for child in parent:
                #if isinstance(child, LTTextLineHorizontal) and isinstance(last_child, LTTextLineHorizontal) and are_similar(child, last_child):
                #    continue
                if isinstance(child, LTChar):
                    yield child
                elif isinstance(child, LTTextBox) or isinstance(child, LTTextLine) or isinstance(child, LTTextLineHorizontal):
                    for child_chars in get_char_objects(child):
                        yield child_chars
                last_child = child

        def parse_layout(layout):
            """Function to recursively parse the layout tree."""
            for lt_obj in layout:
                if isinstance(lt_obj, LTTextBox) or isinstance(lt_obj, LTTextLine):
                    #yield lt_obj.get_text()
                    print("break")
                    for line in lt_obj:
                        print(line.bbox)
                        print(type(line))
                        print(line.get_text())
                        for char in line:
                            if isinstance(char, LTAnno):
                                continue
                            print(char.bbox)
                            print(char.get_text())
                elif isinstance(lt_obj, LTFigure):
                    for string in parse_layout(lt_obj):
                        yield string

        fp = open(filename, 'rb')
        parser = PDFParser(fp)
        doc = PDFDocument(parser)

        rsrcmgr = PDFResourceManager()
        laparams = LAParams()
        device = PDFPageAggregator(rsrcmgr, laparams=laparams)
        interpreter = PDFPageInterpreter(rsrcmgr, device)
        for page in PDFPage.create_pages(doc):
            page_characters = dict()
            interpreter.process_page(page)
            layout = device.get_result()
            for char in get_char_objects(layout):
                row_index = char.bbox[1] // self.row_resolution
                row = page_characters.get(row_index, [])
                if not row:
                    page_characters[row_index] = row
                row.append(char)
            for key in sorted(page_characters, reverse=True):
                string = ""
                last_char = None
                for char in sorted(page_characters[key], key= lambda char: char.bbox[0]):
                    if last_char and last_char.get_text() == char.get_text() and abs(char.bbox[0] - last_char.bbox[0]) < 1:
                        pass
                    elif not last_char or last_char.bbox[2] + 1 > char.bbox[0]:
                        string += char.get_text()
                    else:
                        string += " " + char.get_text()
                    last_char = char
                yield string